﻿''' <summary>
''' Transcription of the GroundOverlay tags
''' </summary>
Public Class GroundOverlay
    '<name>0.png</name>
    '<Region>
    '	<LatLonAltBox>
    '		<north>180</north>
    '		<south>0</south>
    '		<east>0</east>
    '		<west>-180</west>
    '	</LatLonAltBox>
    '	<Lod>
    '		<minLodPixels>128</minLodPixels>
    '		<maxLodPixels>1024</maxLodPixels>
    '	</Lod>
    '</Region>
    '<Icon>
    '	<href>http://earthobservatory.nasa.gov/Feeds/GoogleEarth/ge_44717/0.png</href>
    '</Icon>
    '<LatLonBox>
    '	<north>180</north>
    '	<south>0</south>
    '	<east>0</east>
    '	<west>-180</west>
    '</LatLonBox>
    '<drawOrder>1</drawOrder>
    ''' <summary>
    ''' The name of this groundoverlay tile
    ''' </summary>
    Private _Name As String = String.Empty
    ''' <summary>
    ''' The complete url of this groundoverlay tile
    ''' </summary>
    Private _Url As String = String.Empty
    ''' <summary>
    ''' The draw order of this groundoverlay tile, assimilited to the level of resolution
    ''' </summary>
    Private _drawOrder As Int16 = -1
    ''' <summary>
    ''' The max y coordinate of this groundoverlay tile
    ''' </summary>
    Private _north As Double = -1
    ''' <summary>
    ''' The min y coordinate of this groundoverlay tile
    ''' </summary>
    Private _south As Double = -1
    ''' <summary>
    ''' The max x coordinate of this groundoverlay tile
    ''' </summary>
    Private _east As Double = -1
    ''' <summary>
    ''' The min x coordinate of this groundoverlay tile
    ''' </summary>
    Private _west As Double = -1
    ''' <summary>
    ''' The complete local path including name and extension of the download of this groundoverlay tile
    ''' </summary>
    Private _destinationfile As String = String.Empty
    ''' <summary>
    ''' The local folder in which donwloading this groundoverlay tile, based on the draworder tag
    ''' </summary>
    Private _draworderfolder As String = String.Empty
    ''' <summary>
    ''' The log of errores occured on this groundoverlay tile treatments
    ''' </summary>
    Private _ErrorStatus As String = String.Empty

    ''' <summary>
    ''' Initializes a new instance of the <see cref="GroundOverlay" /> class.
    ''' </summary>
    ''' <param name="groundOverlayNode">The ground overlay node.</param>
    ''' <param name="projectfolder">The project folder path.</param>
    ''' <param name="previousRelativePath">The previous relative path.</param>
    <CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1059:MembersShouldNotExposeCertainConcreteTypes", MessageId:="System.Xml.XmlNode")> _
    <CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")> _
    Public Sub New( _
        ByVal groundOverlayNode As Xml.XmlElement, _
        ByVal projectfolder As String, _
        ByVal previousRelativePath As String)

        Dim urltemp As String
        _north = CDbl(groundOverlayNode.Item("LatLonBox").Item("north").InnerText.ToString.Replace(".", ","))
        _south = CDbl(groundOverlayNode.Item("LatLonBox").Item("south").InnerText.ToString.Replace(".", ","))
        _east = CDbl(groundOverlayNode.Item("LatLonBox").Item("east").InnerText.ToString.Replace(".", ","))
        _west = CDbl(groundOverlayNode.Item("LatLonBox").Item("west").InnerText.ToString.Replace(".", ","))
        _drawOrder = CShort(groundOverlayNode.Item("drawOrder").InnerText.ToString)
        urltemp = groundOverlayNode.Item("Icon").Item("href").InnerText.ToString
        _draworderfolder = System.IO.Path.Combine(projectfolder, _drawOrder.tostring(Globalization.CultureInfo.InvariantCulture.NumberFormat))
        Dim node As Xml.XmlNode = groundOverlayNode.Item("name")
        If Not node Is Nothing Then
            _Name = node.InnerText.ToString
        Else
            _Name = System.Guid.NewGuid().ToString() & Right(urltemp, 4)
        End If


        If _Name = String.Empty Then
            _Name = System.Guid.NewGuid().ToString()
        End If

        If Not Left(urltemp, 4) = "http" Then
            _Url = previousRelativePath & urltemp
        Else
            _Url = urltemp
        End If
        If (IO.Directory.Exists(_draworderfolder)) Then
        Else
            IO.Directory.CreateDirectory(_draworderfolder)
        End If
        _destinationfile = System.IO.Path.Combine(_draworderfolder, _Name)

        Try
            If Not System.IO.File.Exists(_destinationfile) = True Then
                MySingleton.Instance.WC.DownloadFile(_Url, _destinationfile)
                writeTFW()
            End If
        Catch ex As Exception
            _ErrorStatus = String.Format(Globalization.CultureInfo.InvariantCulture, "{0}{1}La tuile {0} n'a pu être enregistrée sous {1} pour la raison {2} {3}", _ErrorStatus, vbNewLine, _Url, _destinationfile, vbNewLine, ex.ToString)
        End Try
    End Sub
    ''' <summary>
    ''' Writes the TFW co-file.
    ''' </summary>
    Private Sub writeTFW()
        'http://georezo.net/forum/viewtopic.php?id=69635
        '- X pixel dimension: Largeur dans l'unité du pixel X
        '- Rotation sur les lignes: 0 pour les bonnes images
        '- Rotation sur les colonnes: 0 pour les bonnes images
        '- Y pixel dimension: Valeur souvent négative (Inverse de l'X pixel dimension, voir ci-dessous)
        '- Coordonnée en X du point d'origine (en principe point supérieur gauche de l'image)
        '- Coordonnée en Y du point d'origine (en principe point supérieur gauche de l'image)
        Dim TFWString As String = String.Empty
        Dim MyImage As System.Drawing.Bitmap = New System.Drawing.Bitmap(Bitmap.FromFile(_destinationfile))
        TFWString = ((_east - _west) / MyImage.Width).ToString(Globalization.CultureInfo.InvariantCulture).Replace(",", ".") & vbNewLine
        TFWString = TFWString & "0" & vbNewLine
        TFWString = TFWString & "0" & vbNewLine
        TFWString = TFWString & "-" & ((_north - _south) / MyImage.Height).ToString(Globalization.CultureInfo.InvariantCulture).Replace(",", ".") & vbNewLine
        TFWString = TFWString & _west.ToString(Globalization.CultureInfo.InvariantCulture).Replace(",", ".") & vbNewLine
        TFWString = TFWString & _north.ToString(Globalization.CultureInfo.InvariantCulture).Replace(",", ".")
        Select Case Right(_destinationfile, 3)
            Case "jpg"
                Using outfile As New IO.StreamWriter(Left(_destinationfile, _destinationfile.Length - 3) & "jgw")
                    outfile.Write(TFWString.ToString())
                End Using
            Case "png"
                Using outfile As New IO.StreamWriter(Left(_destinationfile, _destinationfile.Length - 3) & "pgw")
                    outfile.Write(TFWString.ToString())
                End Using
            Case "tif"
                Using outfile As New IO.StreamWriter(Left(_destinationfile, _destinationfile.Length - 3) & "tfw")
                    outfile.Write(TFWString.ToString())
                End Using
            Case Else
                _ErrorStatus = String.Format(Globalization.CultureInfo.InvariantCulture, "{0}{1}Le géoréférencement de la tuile {0} n'a pu être enregistrée, le format d'image est inconnu", _ErrorStatus, vbNewLine, _Url)
        End Select
        MyImage = nothing
    End Sub
    ''' <summary>
    ''' Gets the error status.
    ''' </summary>
    ''' <value>The error status.</value>
    Public ReadOnly Property ErrorStatus() As String
        Get
            Return _ErrorStatus
        End Get
    End Property
End Class
