﻿''' <summary>
''' Form instance, to write the parameters and launch the processes.
''' </summary>
Public Class KmlExtractorMain
    ''' <summary>
    ''' The project folder
    ''' </summary>
    Private _WorkFolder As String = String.Empty
    ''' <summary>
    ''' The folder used to store locally the kml files
    ''' </summary>
    Private _KmlFolder As String = String.Empty
    ''' <summary>
    ''' Store the percent value of one child kml file rapported on the total childs kml, used to increment the progress bar
    ''' showing the donwload of tiles
    ''' </summary>
    Private _Increment As Double
    ''' <summary>
    ''' Percent of download of tiles
    ''' </summary>
    Private _TileDownloadStatus As Double
    ''' <summary>
    ''' Initializes a new instance of the <see cref="KmlExtractor" /> class.
    ''' </summary>
    Public Sub New()
        InitializeComponent()
        ToolTip.SetToolTip(KmlFilePathLabel, "Fichier kml stocké sur le disque dur duquel extraire les tuiles raster")
        ToolTip.SetToolTip(KmlFilePathTextBox, "Fichier kml stocké sur le disque dur duquel extraire les tuiles raster")
        ToolTip.SetToolTip(projectFolderLabel, "Répertoire dans lequel sera créer le projet")
        ToolTip.SetToolTip(KmlFilePathTextBox, "Répertoire dans lequel sera créer le projet")
        ToolTip.SetToolTip(ProjectNameLabel, "Nom du projet")
        ToolTip.SetToolTip(ProjectNameTextBox, "Nom du projet")
        ToolTip.SetToolTip(RunButton, "Valider les paramètres entrés et lancer le projet")

        KmlFilePathTextBox.Text = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory)
        ProjectFoldertextBox.Text = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)
    End Sub

    ''' <summary>
    ''' Handles the Click event of the RunButton control. Perform check of the inputs and run asynch the
    ''' download of the child kml and kmz files
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
    Private Sub RunButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RunButton.Click
        If AreFormInputsValide() Then
            _WorkFolder = System.IO.Path.Combine(ProjectFoldertextBox.Text, ProjectNameTextBox.Text)
            IO.Directory.CreateDirectory(_WorkFolder)
            _KmlFolder = System.IO.Path.Combine(_WorkFolder, "kml")
            System.IO.Directory.CreateDirectory(_KmlFolder)
            IO.File.Copy(KmlFilePathTextBox.Text, System.IO.Path.Combine(_KmlFolder, "masterkml.kml"), True)
            Me.ConsoleTextBox.Text = "Recherche des fichiers kml enfants"
            KmlKmzFileDownloadBackgroundWorker.RunWorkerAsync()
        Else
            MsgBox("Un ou plusieurs paramètres d'entrée inconsistants")
        End If
    End Sub
    ''' <summary>
    ''' Updates the console.
    ''' </summary>
    ''' <param name="UpdateString">The update string.</param>
    ''' <remarks>Pseudo log Lines are added on the top of the textbox, which is limited to 2000 lines and 100k char</remarks>
    Private Sub UpdateConsole(ByVal UpdateString As String)
        Me.ConsoleTextBox.Text = UpdateString & vbNewLine & Me.ConsoleTextBox.Text
        If CType(Me.ConsoleTextBox.Lines, System.Collections.ICollection).Count = 2001 Then

            'Declare procedural variable  
            Dim lastLine As String = Me.ConsoleTextBox.Lines(CType(Me.ConsoleTextBox.Lines, System.Collections.ICollection).Count - 1)

            'Remove last line  
            If String.IsNullOrEmpty(lastLine) Then
                Me.ConsoleTextBox.Text = Me.ConsoleTextBox.Text.Remove(Me.ConsoleTextBox.Text.LastIndexOf(Environment.NewLine, StringComparison.OrdinalIgnoreCase))
            Else
                Me.ConsoleTextBox.Text = Me.ConsoleTextBox.Text.Remove(Me.ConsoleTextBox.Text.LastIndexOf(lastLine, StringComparison.OrdinalIgnoreCase))
            End If
        End If
    End Sub

    ''' <summary>
    ''' Launch asynch the download of the tiles.
    ''' </summary>
    Private Sub DownloadTiles()
        Me.UpdateConsole("Download des imagettes, jauge d'avancement activée")
        Me.ProgressBar.Visible = True
        Me.ProgressBar.Minimum = 0
        Me.ProgressBar.Maximum = 100
        _Increment = 100 / MySingleton.Instance.KmlKmzFileMasterCollection.Count
        TileDownloadBackgroundWorker.RunWorkerAsync(MySingleton.Instance.KmlKmzFileMasterCollection)
    End Sub
#Region "Input Validation"

    ''' <summary>
    ''' Check if the form controls (so inputs) are valide.
    ''' </summary>
    ''' <returns>True if all inputs are valid</returns>
    Private Function AreFormInputsValide() As Boolean
        Dim validInput As Boolean = True
        If (Not ValidateProjectFoldertextBox()) Or (Not ValidateKmlFilePathTextBox()) Or (Not ValidateProjectNameTextBox()) Then
            validInput = False
        End If
        Return validInput
    End Function
    ''' <summary>
    ''' Handles the Validating event of the ProjectFoldertextBox control.
    ''' </summary>
    ''' <param name="Sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.CancelEventArgs" /> instance containing the event data.</param>
    Private Sub ProjectFoldertextBox_Validating(ByVal Sender As Object, _
                                    ByVal e As System.ComponentModel.CancelEventArgs) Handles _
                                    ProjectFoldertextBox.Validating
        ValidateProjectFoldertextBox()
    End Sub
    ''' <summary>
    ''' Handles the Validating event of the KmlFilePathTextBox control.
    ''' </summary>
    ''' <param name="Sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.CancelEventArgs" /> instance containing the event data.</param>
    Private Sub KmlFilePathTextBox_Validating(ByVal Sender As Object, _
                                    ByVal e As System.ComponentModel.CancelEventArgs) Handles _
                                    KmlFilePathTextBox.Validating
        ValidateKmlFilePathTextBox()
    End Sub
    ''' <summary>
    ''' Handles the Validating event of the ProjectNameTextBox control.
    ''' </summary>
    ''' <param name="Sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.CancelEventArgs" /> instance containing the event data.</param>
    Private Sub ProjectNameTextBox_Validating(ByVal Sender As Object, _
                                    ByVal e As System.ComponentModel.CancelEventArgs) Handles _
                                    ProjectNameTextBox.Validating
        ValidateProjectNameTextBox()
    End Sub

    ''' <summary>
    ''' Validates the project folder textbox.
    ''' </summary>
    ''' <returns>True if the project folder is validated</returns>
    Private Function ValidateProjectFoldertextBox() As Boolean
        Dim ProjectFolderValide As Boolean = True
        If Not IO.Directory.Exists(ProjectFoldertextBox.Text) Then
            ErrorProvider.SetError(ProjectFoldertextBox, "Répertoire inexistant")
            ProjectFolderValide = False
        Else
            ' Clear the error.
            ErrorProvider.SetError(ProjectFoldertextBox, String.Empty)
        End If
        Return ProjectFolderValide
    End Function
    ''' <summary>
    ''' Validates the KML file path textbox.
    ''' </summary>
    ''' <returns>True if the KmlFilePathTextBox is validated</returns>
    Private Function ValidateKmlFilePathTextBox() As Boolean
        Dim KmlFilePathTextBox_Valide As Boolean = True
        If Not IO.File.Exists(KmlFilePathTextBox.Text) Then
            ErrorProvider.SetError(KmlFilePathTextBox, "Fichier kml inexistant sur le disque")
            KmlFilePathTextBox_Valide = False
        Else
            ' Clear the error.
            ErrorProvider.SetError(KmlFilePathTextBox, String.Empty)
        End If
        Return KmlFilePathTextBox_Valide
    End Function
    ''' <summary>
    ''' Validates the project name textbox.
    ''' </summary>
    ''' <returns>True if the ProjectNameTextBox is validated</returns>
    Private Function ValidateProjectNameTextBox() As Boolean
        Dim ProjectNameTextBox_Valide As Boolean = True
        If ProjectNameTextBox.Text.Length = 0 Then
            ErrorProvider.SetError(ProjectNameTextBox, "Nom de projet non fourni")
            ProjectNameTextBox_Valide = False
        Else
            ' Clear the error.
            ErrorProvider.SetError(ProjectNameTextBox, String.Empty)
        End If
        Return ProjectNameTextBox_Valide
    End Function
#End Region
#Region "BackGound Worker for tiles download"
    ''' <summary>
    ''' Handles the DoWork event of the TileDownloadBackgroundWorker control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.DoWorkEventArgs" /> instance containing the event data.</param>
    Private Sub TileDownloadBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles TileDownloadBackgroundWorker.DoWork
        Dim worker As System.ComponentModel.BackgroundWorker = CType(sender, System.ComponentModel.BackgroundWorker)
        e.Result = TileDownloadOperation(TryCast(e.Argument, System.Collections.ObjectModel.Collection(Of KmlKmzFile)), worker, e)
    End Sub

    ''' <summary>
    ''' Handles the ProgressChanged event of the TileDownloadBackgroundWorker control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.ProgressChangedEventArgs" /> instance containing the event data.</param>
    Private Sub TileDownloadBackgroundWorker_ProgressChanged(ByVal sender As System.Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles TileDownloadBackgroundWorker.ProgressChanged
        Me.ProgressBar.Value = e.ProgressPercentage
        Me.UpdateConsole(CType(e.UserState, String))
    End Sub

    ''' <summary>
    ''' Handles the RunWorkerCompleted event of the TileDownloadBackgroundWorker control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.RunWorkerCompletedEventArgs" /> instance containing the event data.</param>
    Private Sub TileDownloadBackgroundWorker_RunWorkerCompleted(ByVal sender As System.Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles TileDownloadBackgroundWorker.RunWorkerCompleted
        Me.ProgressBar.Visible = False
        Me.UpdateConsole("Download des imagettes terminé, jauge d'avancement désactivée")
    End Sub
    ''' <summary>
    ''' Operate the tiles download.
    ''' </summary>
    ''' <param name="KmlKmzFileMasterCollection">The KML KMZ file master collection.</param>
    ''' <param name="worker">The worker.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.DoWorkEventArgs" /> instance containing the event data.</param>
    ''' <returns></returns>
    ''' 
    <CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")> _
    <CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId:="e")> _
    Private Function TileDownloadOperation( _
        ByVal KmlKmzFileMasterCollection As System.Collections.ObjectModel.Collection(Of KmlKmzFile), _
        ByVal worker As System.ComponentModel.BackgroundWorker, _
        ByVal e As System.ComponentModel.DoWorkEventArgs) As Long
        For Each fichier As KmlKmzFile In KmlKmzFileMasterCollection
            fichier.GroundOverlaysSave()
            GC.Collect()
            _TileDownloadStatus = _TileDownloadStatus + _Increment
            worker.ReportProgress(CInt(System.Math.Round(_TileDownloadStatus)), fichier.GroundOverlaysSaveErrors)
        Next
        Return Nothing
    End Function
#End Region
#Region "BackGound Worker for kml & kmz file download"
    ''' <summary>
    ''' Handles the RunWorkerCompleted event of the KmlKmzFileDownloadBackgroundWorker control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.RunWorkerCompletedEventArgs" /> instance containing the event data.</param>
    Private Sub KmlKmzFileDownloadBackgroundWorker_RunWorkerCompleted(ByVal sender As System.Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles KmlKmzFileDownloadBackgroundWorker.RunWorkerCompleted
        DownloadTiles()
    End Sub

    ''' <summary>
    ''' Handles the DoWork event of the KmlKmzFileDownloadBackgroundWorker control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.DoWorkEventArgs" /> instance containing the event data.</param>
    Private Sub KmlKmzFileDownloadBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles KmlKmzFileDownloadBackgroundWorker.DoWork
        Dim worker As System.ComponentModel.BackgroundWorker = CType(sender, System.ComponentModel.BackgroundWorker)
        MySingleton.Instance.KmlKmzFileMasterCollection.Add(New KmlKmzFile(System.IO.Path.Combine(_KmlFolder, "masterkml.kml"), True, _WorkFolder, String.Empty, worker))
    End Sub

    ''' <summary>
    ''' Handles the ProgressChanged event of the KmlKmzFileDownloadBackgroundWorker control.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">The <see cref="System.ComponentModel.ProgressChangedEventArgs" /> instance containing the event data.</param>
    Private Sub KmlKmzFileDownloadBackgroundWorker_ProgressChanged(ByVal sender As System.Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles KmlKmzFileDownloadBackgroundWorker.ProgressChanged
        Me.UpdateConsole(String.Format(Globalization.CultureInfo.InvariantCulture, "     Fichier {0} trouvé et téléchargé", CType(e.UserState, String)))
    End Sub
#End Region
End Class
