﻿''' <summary>
''' Transcription of the href tags
''' </summary>
Public Class KmlKmzFile
    ''' <summary>
    ''' The complete file path to this kml file stored locally as xml file
    ''' </summary>
    Private _FilePath As String = String.Empty
    ''' <summary>
    ''' The complete url of this kml file
    ''' </summary>
    Private _FileUrl As String = String.Empty
    ''' <summary>
    ''' The file name (including extension) of this kml file.
    ''' </summary>
    Private _FileName As String = String.Empty
    ''' <summary>
    ''' The file name of this kml file with kml extension.
    ''' </summary>
    Private _FileNameKML As String = String.Empty
    ''' <summary>
    ''' The file name of this kml file with xml extension.
    ''' </summary>
    Private _FileNameXML As String = String.Empty
    ''' <summary>
    ''' The file name of this kml file with zip extension.
    ''' </summary>
    Private _FileNameZIP As String = String.Empty
    ''' <summary>
    ''' The relative url
    ''' </summary>
    Private _kmlkmzfileref As String = String.Empty
    ''' <summary>
    ''' The actual url path
    ''' </summary>
    Private _kmlkmzhttpfilebase As String = String.Empty
    ''' <summary>
    ''' The project folder
    ''' </summary>
    Private _ProjectFolder As String = String.Empty
    ''' <summary>
    ''' The folder used to store locally the kml files
    ''' </summary>
    Private _KMLFolder As String = String.Empty
    ''' <summary>
    ''' The errors occured while saving the GroundOverlay tiles of this kml file.
    ''' </summary>
    Private _GroundOverlaysSaveErrors As String = String.Empty

    
    ''' <summary>
    ''' Initializes a new instance of the <see cref="KmlKmzFile" /> class.
    ''' </summary>
    ''' <param name="kmlkmzfileref">Le bout d'url relative</param>
    ''' <param name="masterKml">if set to <c>true</c> [master KML].</param>
    ''' <param name="myProjectFolder">Le répertoire du projet sur le client</param>
    ''' <param name="kmlkmzhttpfilebase">L'url de base</param>
    <CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference", MessageId:="4#")> _
    Public Sub New( _
        ByVal kmlkmzfileref As String, _
        ByVal masterKml As Boolean, _
        ByVal myProjectFolder As String, _
        ByVal kmlkmzhttpfilebase As String, _
        ByRef worker As System.ComponentModel.BackgroundWorker)

        _ProjectFolder = String.Format(Globalization.CultureInfo.InvariantCulture, "{0}\", myProjectFolder)
        _KMLFolder = String.Format(Globalization.CultureInfo.InvariantCulture, "{0}kml\", _ProjectFolder)
        _kmlkmzhttpfilebase = kmlkmzhttpfilebase
        If Not masterKml Then
            _kmlkmzfileref = kmlkmzfileref
            'If kmlkmzhttpfilebase = "" Then
            '    _kmlkmzhttpfilebase = "http://earthobservatory.nasa.gov/Feeds/GoogleEarth/ge_44717/"
            'Else
            '    _kmlkmzhttpfilebase = kmlkmzhttpfilebase
            'End If
            If Not Left(kmlkmzfileref, 4) = "http" Then
                _FileUrl = _kmlkmzhttpfilebase & _kmlkmzfileref
            Else
                _FileUrl = _kmlkmzfileref
            End If


            Dim SpliArray As String() = Split(_FileUrl, "/")
            _FileName = SpliArray(SpliArray.Length - 1)
            _FileNameKML = Left(_FileName, _FileName.Length - 3) & "kml"
            _FileNameXML = System.Guid.NewGuid().ToString() & ".xml"
            _FileNameZIP = Left(_FileName, _FileName.Length - 3) & "zip"
            _kmlkmzhttpfilebase = SpliArray(0)
            For i As Int16 = 1 To CShort(CType(SpliArray, System.Collections.ICollection).Count - 2)
                _kmlkmzhttpfilebase = _kmlkmzhttpfilebase & "/" & SpliArray(i)
            Next
            _kmlkmzhttpfilebase = _kmlkmzhttpfilebase & "/"

            'DL du kml/kmz
            If System.IO.File.Exists(_KMLFolder & _FileName) = True Then
                System.IO.File.Delete(_KMLFolder & _FileName)
            End If

            MySingleton.Instance.WC.DownloadFile(_FileUrl, _KMLFolder & _FileName)

            'dezip si nécessaire
            If Right(_FileName, 3) = "kmz" Then
                If System.IO.File.Exists(_ProjectFolder & _FileNameZIP) = True Then
                    System.IO.File.Delete(_ProjectFolder & _FileNameZIP)
                End If
                Dim newName2 As New IO.FileInfo(_KMLFolder & _FileName)
                newName2.MoveTo(_KMLFolder & _FileNameZIP)
                _FileName = _FileNameZIP

                Dim ZipToUnpack As String = _ProjectFolder & _FileNameZIP
                Dim UnpackDirectory As String = _ProjectFolder
                Using zip1 As Ionic.Zip.ZipFile = Ionic.Zip.ZipFile.Read(ZipToUnpack)
                    Dim e As Ionic.Zip.ZipEntry
                    ' here, we extract every entry, but we could extract conditionally,
                    ' based on entry name, size, date, checkbox status, etc.   
                    For Each e In zip1
                        e.Extract(UnpackDirectory, Ionic.Zip.ExtractExistingFileAction.OverwriteSilently)
                    Next
                    zip1.ExtractSelectedEntries(String.Format(Globalization.CultureInfo.InvariantCulture, "name = {0}", _FileNameKML))
                End Using
            End If

            'renommage xml
            _FilePath = _KMLFolder & _FileNameXML
            If System.IO.File.Exists(_FilePath) = True Then
                System.IO.File.Delete(_FilePath)
            End If
            Dim newName As New IO.FileInfo(_KMLFolder & _FileNameKML)
            newName.MoveTo(_KMLFolder & _FileNameXML)
        Else
            _FilePath = kmlkmzfileref
        End If

        NetworkLinksAdding(worker)
        GC.Collect()
        worker.ReportProgress(0, _FileName)
    End Sub

    ''' <summary>
    ''' Launch the download of the GroundOverlays tiles of this kml file.
    ''' </summary>
    Public Sub GroundOverlaysSave()
        Dim XmlDoc As Xml.XmlDocument = New Xml.XmlDocument()
        XmlDoc.Load(_FilePath)
        Dim element As Xml.XmlNodeList
        element = XmlDoc.DocumentElement.GetElementsByTagName("GroundOverlay")
        Dim noeud As Xml.XmlNode
        Dim TileOk As Int16 = 0
        Dim TileError As Int16 = 0
        For Each noeud In element
            Dim MyGroundOverlay As New GroundOverlay(ctype(noeud,Xml.XmlElement), _ProjectFolder, _kmlkmzhttpfilebase)
            If Not MyGroundOverlay.ErrorStatus = String.Empty Then
                _GroundOverlaysSaveErrors = String.Format(Globalization.CultureInfo.InvariantCulture, "{0}{1}{2}{3}", _GroundOverlaysSaveErrors, vbNewLine, "           ", MyGroundOverlay.ErrorStatus)
                TileError = CShort(TileError + 1)
            End If
        Next
        If TileError = 0 Then
            _GroundOverlaysSaveErrors = String.Format(Globalization.CultureInfo.InvariantCulture, "     Pour {0}, {1}{2} {3} tuiles téléchargées sans erreur", _FileName, _GroundOverlaysSaveErrors, vbNewLine, TileOk)
        Else
            _GroundOverlaysSaveErrors = String.Format(Globalization.CultureInfo.InvariantCulture, "     Pour {0}, {1}{2} {3} tuiles téléchargées sans erreur et {4} avec erreur", _FileName, _GroundOverlaysSaveErrors, vbNewLine, TileOk, TileError)
        End If
    End Sub
    ''' <summary>
    ''' Parse the NetworkLinks tags to build new kml files.
    ''' </summary>
    ''' <param name="worker">The worker.</param>
    Private Sub NetworkLinksAdding(ByRef worker As System.ComponentModel.BackgroundWorker)

        Dim XmlDoc As Xml.XmlDocument = New Xml.XmlDocument()
        XmlDoc.Load(_FilePath)
        Dim element As Xml.XmlNodeList
        element = XmlDoc.DocumentElement.GetElementsByTagName("NetworkLink")
        Dim noeud As Xml.XmlNode

        For Each noeud In element
            Dim MyNetworkLink As NetworkLink = New NetworkLink(ctype(noeud,Xml.XmlElement), _kmlkmzhttpfilebase)
            MySingleton.Instance.KmlKmzFileMasterCollection.Add(New KmlKmzFile(MyNetworkLink.GetHref, False, _ProjectFolder, MyNetworkLink.GetHrefBase, worker))
        Next
    End Sub
    ''' <summary>
    ''' Gets the errors occured while saving the GroundOverlay tiles of this kml file.
    ''' </summary>
    ''' <value>The ground overlays save errors.</value>
    Public ReadOnly Property GroundOverlaysSaveErrors() As String
        Get
            Return _GroundOverlaysSaveErrors
        End Get
    End Property
    ''' <summary>
    ''' Gets the name.
    ''' </summary>
    ''' <value>The name of the kml file.</value>
    Public ReadOnly Property Name() As String
        Get
            Return _FileName
        End Get
    End Property    
End Class
