﻿Option Explicit On
Option Strict On
''' <summary>
''' A singleton class to optimize memory and share some objects
''' </summary>
Public Class MySingleton
    Implements IDisposable

    ''' <summary>
    ''' The reference of this singleton instance
    ''' </summary>
    Private Shared _instance As MySingleton
    ''' <summary>
    ''' A webclient used to download files
    ''' </summary>
    Private _wc As New System.Net.WebClient
    ''' <summary>
    ''' The master collection of kml files
    ''' </summary>
    Private _KmlKmzFileMasterCollection As New System.Collections.ObjectModel.Collection(Of KmlKmzFile)
    ''' <summary>
    ''' The number of NetworkLinks
    ''' </summary>
    Private _NetworkLinkCount As Integer
    ''' <summary>
    ''' Initializes a new instance of the <see cref="MySingleton" /> class.
    ''' Marked as private to provide singleton mechanism.
    ''' </summary>
    Private Sub New()
        '
    End Sub

    ''' <summary>
    ''' Gets the instance. Marked as shared to provide singleton mechanism.
    ''' </summary>
    ''' <returns>The singleton instance</returns>
    Public Shared Function Instance() As MySingleton

        If _instance Is Nothing Then
            _instance = New MySingleton
        End If

        Return _instance

    End Function
    ''' <summary>
    ''' Gets or sets the network link count.
    ''' </summary>
    ''' <value>The network link count.</value>
    Public Property NetworkLinkCount() As Integer
        Get
            Return _NetworkLinkCount
        End Get
        Set(ByVal value As Integer)
            _NetworkLinkCount = value
        End Set
    End Property
    ''' <summary>
    ''' Gets the WebClient.
    ''' </summary>
    ''' <value>The WebClient.</value>
    Public ReadOnly Property WC() As System.Net.WebClient
        Get
            Return _wc
        End Get
    End Property
    ''' <summary>
    ''' Gets or sets the master collection of kml files.
    ''' </summary>
    ''' <value>The master collection of kml files.</value>
    Public ReadOnly Property KmlKmzFileMasterCollection() As System.Collections.ObjectModel.Collection(Of KmlKmzFile)
        Get
            Return _KmlKmzFileMasterCollection
        End Get
    End Property
#Region "Fix rule CA1001 of fxcop"
    Protected Overridable Overloads Sub Dispose(ByVal disposing As Boolean)

        If disposing Then
            ' dispose managed resources
            WC.Dispose()
        End If

        ' free native resources

    End Sub 'Dispose
    Public Overloads Sub Dispose() Implements IDisposable.Dispose

        Dispose(True)
        GC.SuppressFinalize(Me)

    End Sub 'Dispose

#End Region

End Class


